package nl.uvt.locator;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;

public class QueryData {
	String genre;   
	String issn;
	String isbn;
	String title;
	String atitle;
	String epn;
	String database;
	String oIssue;
	String location;
	int year;
	int volume;
	int issue;
	String spage;
	QueryData(){
		genre = "article";
		issn = "";
		isbn = "";
		title = "";
		atitle = "";
		oIssue = "";
		epn = "";
		database = "";
		year = -1;
		volume = -1;
		issue = -1;
		spage = "";
		location = "";
	}

	public String toString(){
		String result = "genre=" + genre;
		result += ", title=" + title;
		result += ", atitle=" + atitle;
		result += ", issn=" + issn;
		result += ", isbn=" + isbn;
		result += ", database="+ database;
		result += ", epn="+ epn;
		result += ", year="+ year;
		result += ", volume=" + volume;
		result += ", issue=" + issue;
		result += ", location=" + location;
		result += ", origIssue=" + oIssue;
		result += ", spage=" + spage;
		return result;
	}

	public boolean parseQuery( HttpServletRequest rec ) {
		String query = rec.getQueryString();
		if ( query != null ){
			Locator.logger.debug( "query: " + query );
			String [] attVals = query.split("&");
			Map<String,String> parameters = null;
			if ( attVals.length > 0 ){
				parameters = new HashMap<String,String>();
				for( int i=0; i < attVals.length; ++i ){
					String [] parts = attVals[i].split("=");
					if ( parts.length > 1 ){
						try {
							String val = URLDecoder.decode( parts[1], "UTF8");
							parameters.put(parts[0], val );
						} catch (UnsupportedEncodingException e) {
							Locator.logger.debug(e);
						}
					}
				}
				String par = parameters.get("genre" );
				if ( par != null && par.length() > 0 )
					genre = par;
				par = parameters.get( "issn" );
				if ( par != null && par.length() > 0 ){
					issn = par.trim();
					issn = issn.replaceAll("-", "");
				}
				par = parameters.get( "isbn" );
				if ( par != null && par.length() > 0 ){
					isbn = par.trim();
					isbn = isbn.replaceAll("-", "");
				}
				par = parameters.get( "location" );
				if ( par != null && par.length() > 0 )
					location = par;
				par = parameters.get( "title" );
				if ( par != null && par.length() > 0 )
					title = par;
				par = parameters.get( "atitle" );
				if ( par != null && par.length() > 0 ){
					atitle = par;
				}
				par = parameters.get( "epn" );
				if ( par != null && par.length() > 0 )
					epn = par;
				par = parameters.get( "oissue" );
				if ( par != null && par.length() > 0 )
					oIssue = par;
				par = parameters.get( "spage" );
				if ( par != null && par.length() > 0 ){
					Pattern braces = Pattern.compile("^\\d+");
					Matcher myMatch = braces.matcher(par);
					String match = "";
					if ( myMatch.find() ){
						match = myMatch.group(0);
						Locator.logger.debug( "final spages match = " + match );
						spage = match;
					}
					else
						spage	 = par;
				}
				try {
					par = parameters.get( "year" );
					if ( par != null && par.length() > 0 )
						year = Integer.parseInt( par );
					par = parameters.get( "date" );
					if ( par != null && par.length() > 0 )
						year = Integer.parseInt( par );
					par = parameters.get( "volume" );
					if ( par != null && par.length() > 0 ){
						// try to compensate for volumes like: '11 Suppl 1'
						String[] parts = par.split(" ");
						if ( parts.length > 1 ){
							Locator.logger.warn("Guessing volume from '" + par + "'. Try '" + parts[0] + "' to determine Volume." );
							volume = Integer.parseInt( parts[0] );
						}
						else
							volume = Integer.parseInt( par );
					}
					par = parameters.get( "issue" );
					if ( par != null && par.length() > 0 )
						issue = Integer.parseInt( par );
				}
				catch( NumberFormatException e ){
					Locator.logger.debug( "Integer.parse() failed on:" + par );
					return false;
				}
			}
		}
		return true;
	}

}
